package com.aliyun.vodplayer.demo.activity.advance;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.view.Display;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ListView;
import android.widget.Toast;

import com.alivc.player.VcPlayerLog;
import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.activity.advance.download.AddDownloadView;
import com.aliyun.vodplayer.demo.activity.advance.download.DownloadAdapter;
import com.aliyun.vodplayer.demo.utils.BaseAppCompatActivity;
import com.aliyun.vodplayer.demo.utils.VidStsUtil;
import com.aliyun.vodplayer.downloader.AliyunDownloadInfoListener;
import com.aliyun.vodplayer.downloader.AliyunDownloadManager;
import com.aliyun.vodplayer.downloader.AliyunDownloadMediaInfo;
import com.aliyun.vodplayer.downloader.AliyunRefreshStsCallback;
import com.aliyun.vodplayer.media.AliyunVidSts;
import com.aliyun.vodplayerview.utils.NetWatchdog;

import java.lang.ref.WeakReference;
import java.util.List;

public class DemoDownloadActivity extends BaseAppCompatActivity {

    private static final String TAG = DemoDownloadActivity.class.getSimpleName();
    private ListView downloadListView;
    private DownloadAdapter downloadAdapter;
    private AliyunDownloadManager downloadManager;
    private NetWatchdog netWatchdog;
    private AliyunDownloadInfoListener downloadInfoListener;


    //    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_demo_download);

        findAllViews();
        downloadInfoListener = new MyDownloadInfoListener(this);
        downloadManager = AliyunDownloadManager.getInstance(this);
        downloadManager.addDownloadInfoListener(downloadInfoListener);
//        downloadManager.setRefreshAuthCallBack(authCallback);
        downloadManager.setRefreshStsCallback(new MyRefreshStsCallback());

        downloadAdapter = new DownloadAdapter(this);
        downloadListView.setAdapter(downloadAdapter);

        final List<AliyunDownloadMediaInfo> downloadingMedias = downloadManager.getDownloadingMedias();
        downloadAdapter.addAllInfos(downloadingMedias);

        netWatchdog = new NetWatchdog(this);
        netWatchdog.setNetChangeListener(new NetWatchdog.NetChangeListener() {
            @Override
            public void onWifiTo4G() {
                if(hasDownloading()) {
                    downloadManager.stopDownloadMedias(downloadAdapter.getDownloadMedias());
                    showNetChangeDialog();
                }
            }

            @Override
            public void on4GToWifi() {

            }

            @Override
            public void onNetDisconnected() {

            }
        });

        netWatchdog.startWatch();
    }

    private boolean hasDownloading() {
        List<AliyunDownloadMediaInfo> downloadingMedias = downloadManager.getDownloadingMedias();
        for(AliyunDownloadMediaInfo info : downloadingMedias){
            if(info.getStatus() == AliyunDownloadMediaInfo.Status.Start ||
                    info.getStatus() == AliyunDownloadMediaInfo.Status.Wait){
                return true;
            }
        }
        return false;
    }


    private AlertDialog netChangeDialog = null;

    private void showNetChangeDialog() {
        if (netChangeDialog == null) {
            AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
            dialogBuilder.setTitle(R.string.net_change_to_4g);
            dialogBuilder.setMessage(R.string.continue_to_download);
            dialogBuilder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    netChangeDialog = null;
                    dialog.dismiss();

                    List<AliyunDownloadMediaInfo> downloadMediaInfos = downloadAdapter.getDownloadMedias();
                    for (AliyunDownloadMediaInfo info : downloadMediaInfos) {
                        downloadManager.startDownloadMedia(info);
                    }
                }
            });
            dialogBuilder.setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {

                    netChangeDialog = null;
                    dialog.dismiss();
                }
            });
            netChangeDialog = dialogBuilder.create();
        }

        if (!netChangeDialog.isShowing()) {
            netChangeDialog.show();
        }
    }


    @Override
    protected void onDestroy() {
        //移除防止内存泄漏
        downloadManager.removeDownloadInfoListener(downloadInfoListener);
        netWatchdog.stopWatch();


        super.onDestroy();
    }

    private void findAllViews() {
        downloadListView = (ListView) findViewById(R.id.download_list);

        findViewById(R.id.add_download).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showAddDownloadView();
            }
        });

    }


    private void showAddDownloadView() {
        Dialog dialog = new Dialog(this/*,R.style.addDownloadDialog*/);
        final AddDownloadView contentView = new AddDownloadView(this);

        dialog.setTitle("选择下载项");
        contentView.setOnViewClickListener(viewClickListener);
        dialog.setContentView(contentView);
/*
         * 将对话框的大小按屏幕大小的百分比设置
         */
        WindowManager m = getWindowManager();
        Display d = m.getDefaultDisplay(); // 获取屏幕宽、高用
        Window dialogWindow = dialog.getWindow();
        WindowManager.LayoutParams p; // 获取对话框当前的参数值
        if (dialogWindow != null) {
            p = dialogWindow.getAttributes();
            p.width = (int) (d.getWidth() * 0.96); // 宽度设置
            dialogWindow.setAttributes(p);
        }

        dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialogInterface) {
                contentView.destory();
            }
        });
        dialog.show();
    }

    private Dialog dialog = null;

    private AddDownloadView.OnViewClickListener viewClickListener = new AddDownloadView.OnViewClickListener() {
        @Override
        public void onCancel() {
            if (dialog != null) {
                dialog.dismiss();
            }
        }

        @Override
        public void onDownload(AliyunDownloadMediaInfo info) {
            if (dialog != null) {
                dialog.dismiss();
            }
            addNewInfo(info);
        }
    };

    private void addNewInfo(AliyunDownloadMediaInfo info) {
        downloadManager.addDownloadMedia(info);
        downloadAdapter.addNewInfo(info);
    }

    private static class MyDownloadInfoListener implements AliyunDownloadInfoListener {

        private WeakReference<DemoDownloadActivity> weakActivity;

        public MyDownloadInfoListener(DemoDownloadActivity activity) {
            weakActivity = new WeakReference<DemoDownloadActivity>(activity);
        }

        @Override
        public void onPrepared(List<AliyunDownloadMediaInfo> infos) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onPrepared(infos);
            }
        }

        @Override
        public void onStart(AliyunDownloadMediaInfo info) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onStart(info);
            }
        }

        @Override
        public void onProgress(AliyunDownloadMediaInfo info, int percent) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onProgress(info, percent);
            }
        }

        @Override
        public void onStop(AliyunDownloadMediaInfo info) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onStop(info);
            }
        }

        @Override
        public void onCompletion(AliyunDownloadMediaInfo info) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onCompletion(info);
            }
        }

        @Override
        public void onError(AliyunDownloadMediaInfo info, int code, String msg, String reuqestId) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onError(info, code, msg, reuqestId);
            }
        }

        @Override
        public void onWait(AliyunDownloadMediaInfo outMediaInfo) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onWait(outMediaInfo);
            }
        }

        @Override
        public void onM3u8IndexUpdate(AliyunDownloadMediaInfo outMediaInfo, int index) {
            DemoDownloadActivity downloadActivity = weakActivity.get();
            if (downloadActivity != null) {
                downloadActivity.onM3u8IndexUpdate(outMediaInfo, index);
            }
        }
    }


    private void onPrepared(List<AliyunDownloadMediaInfo> list) {

    }


    private void onStart(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {
        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
    }


    private void onProgress(AliyunDownloadMediaInfo aliyunDownloadMediaInfo, int i) {

        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
    }


    private void onStop(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {

        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
    }


    private void onCompletion(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {

        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
    }


    private void onError(AliyunDownloadMediaInfo aliyunDownloadMediaInfo, int i, String s, String requestId) {

        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
        if (i == 107) {
            //网络异常
            Toast.makeText(DemoDownloadActivity.this.getApplicationContext(), R.string.net_disconnect, Toast.LENGTH_LONG).show();
        } else {
            Toast.makeText(DemoDownloadActivity.this.getApplicationContext(), s, Toast.LENGTH_LONG).show();
        }
    }


    private void onWait(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {
        downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
        downloadAdapter.notifyDataSetChanged();
    }


    private void onM3u8IndexUpdate(AliyunDownloadMediaInfo outMediaInfo, int index) {

    }


//    private AliyunRefreshPlayAuthCallback authCallback = new AliyunRefreshPlayAuthCallback() {
//        //                        @RequiresApi(api = Build.VERSION_CODES.KITKAT)
//        @Override
//        public AliyunPlayAuth refreshPlayAuth(String vid, String quality, String format, String title, boolean encript) {
//            //NOTE: 注意：这个不能启动线程去请求。因为这个方法已经在线程中调用了。
//            VcPlayerLog.d(TAG,"refreshSts , vid = " + vid );
//            String playauth = PlayAuthUtil.getPlayAuth( vid);
//            if (playauth == null) {
//                return null;
//            }
//            AliyunPlayAuth.AliyunPlayAuthBuilder authBuilder = new AliyunPlayAuth.AliyunPlayAuthBuilder();
//            authBuilder.setPlayAuth(playauth);
//            authBuilder.setVid(vid);
//            authBuilder.setTitle(title);
//            authBuilder.setQuality(quality);
//            authBuilder.setFormat(format);
//            authBuilder.setIsEncripted(encript ? 1 : 0);
//            return authBuilder.build();
//        }
//    };

    private static class MyRefreshStsCallback implements AliyunRefreshStsCallback {

        @Override
        public AliyunVidSts refreshSts(String vid, String quality, String format, String title, boolean encript) {
            VcPlayerLog.d("refreshSts ", "refreshSts , vid = " + vid);
            //NOTE: 注意：这个不能启动线程去请求。因为这个方法已经在线程中调用了。
            AliyunVidSts vidSts = VidStsUtil.getVidSts();
            if (vidSts == null) {
                return null;
            } else {
                vidSts.setVid(vid);
                vidSts.setQuality(quality);
                vidSts.setTitle(title);
                return vidSts;
            }
        }
    }

}
