package com.aliyun.vodplayer.demo.activity.advance;

import android.graphics.Color;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.media.AliyunLocalSource;
import com.aliyun.vodplayer.media.AliyunVodPlayer;
import com.aliyun.vodplayer.media.IAliyunVodPlayer;

/**
 *
 */

public class DemoSwitchFragment extends Fragment {


    public static final String LIVEURL = "liveurl";
    private static final String TAG = "DemoSwitchFragment";
    boolean mIsVisibleToUser;// 界面是否可见
    boolean mCreateView;// 界面是否创建
    public String mLiveUrl;
    public int mPosition;
    private View mPlayingView;
    private FrameLayout mSurfaceContainer;
    private SurfaceView mSurfaceView;
    public static AliyunVodPlayer aliyunVodPlayer = null;

    public DemoSwitchFragment() {
    }


    public static DemoSwitchFragment newInstance(int position) {
        DemoSwitchFragment newFragment = new DemoSwitchFragment();
        newFragment.mPosition = position;
        return newFragment;

    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        mLiveUrl = getArguments().getString(LIVEURL);
        mPlayingView = inflater.inflate(R.layout.fragment_demo_switch, null);
        mSurfaceContainer = (FrameLayout) mPlayingView.findViewById(R.id.liveplaying_surfaceview);
        Log.d(TAG, "onCreateView==" + mPosition);
        return mPlayingView;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mCreateView = true;
        if (mIsVisibleToUser) {
//            //加载数据
            Log.d(TAG, "load data==" + mPosition);
            initSurface();
        }
        Log.d(TAG, "onActivityCreated==" + mPosition);
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        Log.d(TAG, "setUserVisibleHint=" + isVisibleToUser + "====" + mPosition);
        mIsVisibleToUser = isVisibleToUser;
        if (!mIsVisibleToUser && mPlayingView != null && aliyunVodPlayer != null) {
            //释放数据
            Log.d(TAG, "release data==" + mPosition);
            aliyunVodPlayer.stop();
            aliyunVodPlayer.release();
            aliyunVodPlayer = null;
            removeSurface();
        } else if (mIsVisibleToUser && mCreateView) {
//            //加载数据
            Log.d(TAG, "load data==" + mPosition);
            initSurface();
        }
    }

    @Override
    public void onDestroyView() {
        if (mIsVisibleToUser && aliyunVodPlayer != null
                ) {
            Log.d(TAG, "release data==" + mPosition);
            //释放数据
            aliyunVodPlayer.stop();
            aliyunVodPlayer.release();
            aliyunVodPlayer = null;
            removeSurface();
        }
        mCreateView = false;
        mPlayingView = null;
        mIsVisibleToUser = false;
        super.onDestroyView();
        Log.d(TAG, "onDestroyView==" + mPosition);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.d(TAG, "onDestroy==" + mPosition);
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.d(TAG, "onStart==" + mPosition);
    }

    @Override
    public void onStop() {
        super.onStop();
        Log.d(TAG, "onStop==" + mPosition);
    }

    /**
     * 重点:初始化播放器使用的SurfaceView,此处的SurfaceView采用动态添加
     *
     * @return 是否成功
     */
    private boolean initSurface() {
        mSurfaceContainer.setBackgroundColor(Color.rgb(0, 0, 0));
        mSurfaceView = new SurfaceView(getContext());

        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT,
                FrameLayout.LayoutParams.MATCH_PARENT);
        params.gravity = Gravity.CENTER;
        mSurfaceView.setLayoutParams(params);
        // 为避免重复添加,事先remove子view
        mSurfaceContainer.removeAllViews();
        mSurfaceContainer.addView(mSurfaceView);

        mSurfaceView.setZOrderOnTop(false);


        mSurfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                Log.d(TAG, "surfaceCreated" + DemoSwitchFragment.this);
                holder.setType(SurfaceHolder.SURFACE_TYPE_GPU);
                holder.setKeepScreenOn(true);

                AliyunVodPlayer tmpVodPlayer = aliyunVodPlayer;
                if (tmpVodPlayer != null) {
                    tmpVodPlayer.stop();
                    tmpVodPlayer.release();
                }
                aliyunVodPlayer = null;

                initVodPlayer();

                String url = mLiveUrl;
                AliyunLocalSource.AliyunLocalSourceBuilder asb = new AliyunLocalSource.AliyunLocalSourceBuilder();
                asb.setSource(url);

                aliyunVodPlayer.setDisplay(holder);
                aliyunVodPlayer.prepareAsync(asb.build());
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
                Log.d(TAG, "surfaceChanged" + DemoSwitchFragment.this);
                aliyunVodPlayer.surfaceChanged();
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                Log.d(TAG, "surfaceDestroyed" + DemoSwitchFragment.this);
            }
        });

        return true;
    }

    private void removeSurface() {
        mSurfaceContainer.removeAllViews();
        mSurfaceView = null;
    }

    private void initVodPlayer() {
        aliyunVodPlayer = new AliyunVodPlayer(getContext());
        aliyunVodPlayer.setOnPreparedListener(new IAliyunVodPlayer.OnPreparedListener() {
            @Override
            public void onPrepared() {
                //准备成功之后可以调用start方法开始播放
                aliyunVodPlayer.start();
            }
        });
        aliyunVodPlayer.setOnErrorListener(new IAliyunVodPlayer.OnErrorListener() {
            @Override
            public void onError(int arg0, int arg1, String msg) {
            }
        });
        aliyunVodPlayer.setOnCompletionListener(new IAliyunVodPlayer.OnCompletionListener() {
            @Override
            public void onCompletion() {
                Log.d(TAG, "onCompletion--- ");
            }
        });
        aliyunVodPlayer.setOnBufferingUpdateListener(new IAliyunVodPlayer.OnBufferingUpdateListener() {
            @Override
            public void onBufferingUpdate(int percent) {
                Log.d(TAG, "onBufferingUpdate--- " + percent);
            }
        });
        aliyunVodPlayer.setOnChangeQualityListener(new IAliyunVodPlayer.OnChangeQualityListener() {
            @Override
            public void onChangeQualitySuccess(String finalQuality) {
                Log.d(TAG, "onChangeQualitySuccess");

                aliyunVodPlayer.changeQuality(IAliyunVodPlayer.QualityValue.QUALITY_FLUENT);
            }

            @Override
            public void onChangeQualityFail(int code, String msg) {
                Log.d(TAG, "onChangeQualityFail。。。" + code + " ,  " + msg);
            }
        });
        aliyunVodPlayer.enableNativeLog();


    }
}
